@@ -83,8 +83,15 @@ class DatasetDescriptor:
 
     embedding_column: Optional[str] = None
 
+    # only when the embedding column is a map
+    embedding_column_key: Optional[Any] = None
+
     embedding_id_column: Optional[str] = None
 
+    # filters on the dataset where each filter is a
+    # string rep of a filter expression
+    filters: Optional[List[str]] = None
+
     # unused in open-source
     splits_distribution: Optional[List[List[bytes]]] = None
 
@@ -106,6 +113,10 @@ class DatasetDescriptor:
     # desc_name
     desc_name: Optional[str] = None
 
+    filename_suffix: Optional[str] = None
+
+    normalize_L2: bool = False
+
     def __hash__(self):
         return hash(self.get_filename())
 
@@ -129,6 +140,8 @@ class DatasetDescriptor:
             ).replace("=", "_").replace("/", "_")
         if self.num_vectors is not None:
             filename += f"_{self.num_vectors}"
+        if self.filename_suffix is not None:
+            filename += f"_{self.filename_suffix}"
         filename += "."
 
         self.desc_name = filename
@@ -214,6 +227,8 @@ class CodecDescriptor(IndexBaseDescriptor):
     factory: Optional[str] = None
     construction_params: Optional[List[Dict[str, int]]] = None
     training_vectors: Optional[DatasetDescriptor] = None
+    normalize_l2: bool = False
+    is_spherical: bool = False
     FILENAME_PREFIX: str = "xt"
 
     def __post_init__(self):
