@@ -1,19 +1,33 @@
-/**
- * Copyright (c) Facebook, Inc. and its affiliates.
+/*
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * This source code is licensed under the MIT license found in the
  * LICENSE file in the root directory of this source tree.
  */
 
-// Copyright 2004-present Facebook. All Rights Reserved.
 // -*- c++ -*-
 
 #include "Index_c.h"
 #include <faiss/Index.h>
+#include <faiss/impl/IDSelector.h>
 #include "macros_impl.h"
 
 extern "C" {
 
+DEFINE_DESTRUCTOR(SearchParameters)
+
+int faiss_SearchParameters_new(
+        FaissSearchParameters** p_sp,
+        FaissIDSelector* sel) {
+    try {
+        faiss::SearchParameters* params = new faiss::SearchParameters;
+        params->sel = reinterpret_cast<faiss::IDSelector*>(sel);
+        *p_sp = reinterpret_cast<FaissSearchParameters*>(params);
+        return 0;
+    }
+    CATCH_AND_HANDLE
+}
+
 DEFINE_DESTRUCTOR(Index)
 
 DEFINE_GETTER(Index, int, d)
@@ -66,6 +80,26 @@ int faiss_Index_search(
     CATCH_AND_HANDLE
 }
 
+int faiss_Index_search_with_params(
+        const FaissIndex* index,
+        idx_t n,
+        const float* x,
+        idx_t k,
+        const FaissSearchParameters* params,
+        float* distances,
+        idx_t* labels) {
+    try {
+        reinterpret_cast<const faiss::Index*>(index)->search(
+                n,
+                x,
+                k,
+                distances,
+                labels,
+                reinterpret_cast<const faiss::SearchParameters*>(params));
+    }
+    CATCH_AND_HANDLE
+}
+
 int faiss_Index_range_search(
         const FaissIndex* index,
         idx_t n,
