@@ -1,4 +1,4 @@
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -18,7 +18,7 @@ class TestGpuIndexIvfflat(unittest.TestCase):
         res = faiss.StandardGpuResources()
         res.noTempMemory()
         config = faiss.GpuIndexIVFFlatConfig()
-        config.use_raft = False
+        config.use_cuvs = False
         index2 = faiss.GpuIndexIVFFlat(res, index, config)
         recons = index2.reconstruct_n(0, 10)
 
