@@ -15,9 +15,11 @@
  * the interface.
  */
 
-#include <faiss/MetricType.h>
 #include <vector>
 
+#include <faiss/MetricType.h>
+#include <faiss/impl/maybe_owned_vector.h>
+
 namespace faiss {
 
 struct InvertedListsIterator {
@@ -241,8 +243,8 @@ struct InvertedLists {
 
 /// simple (default) implementation as an array of inverted lists
 struct ArrayInvertedLists : InvertedLists {
-    std::vector<std::vector<uint8_t>> codes; // binary codes, size nlist
-    std::vector<std::vector<idx_t>> ids;     ///< Inverted lists for indexes
+    std::vector<MaybeOwnedVector<uint8_t>> codes; // binary codes, size nlist
+    std::vector<MaybeOwnedVector<idx_t>> ids; ///< Inverted lists for indexes
 
     ArrayInvertedLists(size_t nlist, size_t code_size);
 
