@@ -24,6 +24,8 @@ TEST(TestFactoryTools, TestReverseIndexFactory) {
                  "HNSW32",
                  "SQ8",
                  "SQfp16",
+                 "NSG24,Flat",
+                 "NSG16,SQ8",
          }) {
         std::unique_ptr<Index> index{index_factory(64, factory)};
         ASSERT_TRUE(index);
@@ -32,6 +34,8 @@ TEST(TestFactoryTools, TestReverseIndexFactory) {
     using Case = std::pair<const char*, const char*>;
     for (auto [src, dst] : {
                  Case{"SQ8,RFlat", "SQ8,Refine(Flat)"},
+                 Case{"NSG", "NSG32,Flat"},
+                 Case{"NSG,PQ8", "NSG32,PQ8x8"},
          }) {
         std::unique_ptr<Index> index{index_factory(64, src)};
         ASSERT_TRUE(index);
